@@ -18,13 +18,7 @@
 package AI::MXNet::Contrib::NDArray;
 use strict;
 use warnings;
-
-sub AUTOLOAD {
-    my $sub = $AI::MXNet::Contrib::NDArray::AUTOLOAD;
-    $sub =~ s/.*:://;
-    $sub = "_contrib_$sub";
-    shift;
-    return AI::MXNet::NDArray->$sub(@_);
-}
+use parent 'AI::MXNet::AutoLoad';
+sub config { ('contrib', 'AI::MXNet::NDArray') }
 
 1;
